"""Main metron_tagger tests"""
# import io
# import sys
import sys
from pathlib import Path
from zipfile import ZipFile

import pytest
from darkseid.comic import Comic

from metrontagger.run import Runner
from metrontagger.settings import MetronTaggerSettings
from metrontagger.talker import Talker

# from darkseid.metadata import Metadata
# from mokkari.issue import IssuesList


def test_create_metron_talker(tmp_path: Path) -> None:
    s = MetronTaggerSettings(tmp_path)
    s.metron_user = "test"
    s.metron_pass = "test_password"
    talker = Talker(s.metron_user, s.metron_pass)
    assert isinstance(talker, Talker)


@pytest.mark.skipif(sys.platform in ["win32"], reason="Skip Windows.")
def test_export_to_cb7(fake_comic: ZipFile, tmpdir) -> None:
    # This function will create a new archive with a cb7 extension.
    r = Runner(MetronTaggerSettings(tmpdir))
    r._export_to_cb7([fake_comic])
    new_name = Path(fake_comic).with_suffix(".cb7")
    ca = Comic(new_name)
    assert ca.is_sevenzip() is True
    assert ca.get_number_of_pages() == 3


# def test_list_comics_with_missing_metadata(fake_comic: ZipFile) -> None:
#     expected_result = (
#         "\nShowing files without metadata:"
#         + "\n-------------------------------"
#         + "\nno metadata in 'Aquaman v1 #001 (of 08) (1994).cbz'"
#         + "\n"
#     )
#     # Make sure fake comic archive doesn't have any metadata
#     if Comic(fake_comic).has_metadata():
#         Comic(fake_comic).remove_metadata()

#     fake_list = [fake_comic]

#     # Capture the output so we can verify the print output
#     captured_output = io.StringIO()
#     sys.stdout = captured_output

#     list_comics_with_missing_metadata(fake_list)
#     sys.stdout = sys.__stdout__

#     assert expected_result == captured_output.getvalue()


# def test_delete_comics_with_metadata(
#     fake_comic: ZipFile, fake_metadata: Metadata
# ) -> None:
#     expected_result = (
#         "\nRemoving metadata:\n-----------------"
#         + "\nremoved metadata from 'Aquaman v1 #001 (of 08) (1994).cbz'"
#         + "\n"
#     )

#     # If comic doesn't already have metadata let's add it
#     if not Comic(fake_comic).has_metadata():
#         Comic(fake_comic).write_metadata(fake_metadata)

#     fake_list = [fake_comic]

#     # Capture the output so we can verify the print output
#     captured_output = io.StringIO()
#     sys.stdout = captured_output

#     delete_comics_metadata(fake_list)
#     sys.stdout = sys.__stdout__

#     assert expected_result == captured_output.getvalue()


# def test_delete_comics_without_metadata(fake_comic: ZipFile) -> None:
#     expected_result = (
#         "\nRemoving metadata:\n-----------------"
#         + "\nno metadata in 'Aquaman v1 #001 (of 08) (1994).cbz'"
#         + "\n"
#     )

#     # If comic has metadata let's remove it
#     if Comic(fake_comic).has_metadata():
#         Comic(fake_comic).remove_metadata()

#     fake_list = [fake_comic]

#     # Capture the output so we can verify the print output
#     captured_output = io.StringIO()
#     sys.stdout = captured_output

#     delete_comics_metadata(fake_list)
#     sys.stdout = sys.__stdout__

#     assert expected_result == captured_output.getvalue()


# def test_sort_comics_without_sort_dir(fake_comic: ZipFile, tmp_path: Path) -> None:
#     expected_result = "\nUnable to sort files. No destination directory was provided.\n"

#     # Create fake settings.
#     s = MetronTaggerSettings(tmp_path)
#     s.sort_dir = ""

#     fake_list = [fake_comic]

#     # Capture the output so we can verify the print output
#     captured_output = io.StringIO()
#     sys.stdout = captured_output

#     sort_list_of_comics(s.sort_dir, fake_list)
#     sys.stdout = sys.__stdout__

#     assert expected_result == captured_output.getvalue()


# def test_sort_comics_with_dir(
#     fake_comic: ZipFile, fake_metadata: Metadata, tmp_path: Path
# ) -> None:
#     s = MetronTaggerSettings(tmp_path)
#     s.sort_dir = tmp_path

#     res_path = (
#         Path(f"{s.sort_dir}")
#         / f"{fake_metadata.publisher}"
#         / f"{fake_metadata.series}"
#         / f"v{fake_metadata.volume}"
#     )

#     expected_result = (
#         "\nStarting sorting of comic archives:\n----------------------------------\n"
#         + "moved 'Aquaman v1 #001 (of 08) (1994).cbz' to "
#         + "'"
#         + str(res_path)
#         + "'\n"
#     )

#     comic = Comic(fake_comic)

#     if not comic.has_metadata():
#         comic.write_metadata(fake_metadata)

#     fake_list = [fake_comic]

#     # Capture the output so we can verify the print output
#     captured_output = io.StringIO()
#     sys.stdout = captured_output

#     sort_list_of_comics(s.sort_dir, fake_list)
#     sys.stdout = sys.__stdout__

#     # Path for moved file
#     moved_comic = (
#         Path(f"{s.sort_dir}")
#         / f"{fake_metadata.publisher}"
#         / f"{fake_metadata.series}"
#         / f"v{fake_metadata.volume}"
#         / fake_comic.name
#     )

#     assert moved_comic.parent.is_dir()
#     assert moved_comic.is_file()
#     assert expected_result == captured_output.getvalue()


# @pytest.fixture()
# def multi_choice_fixture() -> IssuesList:
#     i = {
#         "count": 2,
#         "next": None,
#         "previous": None,
#         "results": [
#             {"issue": "Superman #1", "cover_date": "1939-10-01"},
#             {"issue": "Superman #1", "cover_date": "1986-01-01"},
#         ],
#     }
#     return IssuesList(i)


# def test_print_multi_choices_to_user(
#     talker: Talker, multi_choice_fixture: IssuesList, capsys
# ) -> None:
#     fn = Path("/tmp/Superman #1")
#     test_data = MultipleMatch(fn, multi_choice_fixture)
#     expected_result = "1. Superman #1 (1939-10-01)\n2. Superman #1 (1986-01-01)\n"
#     talker._print_choices_to_user(test_data.matches)
#     stdout, _ = capsys.readouterr()
#     assert stdout == expected_result


# def test_post_process_matches(capsys, talker: Talker) -> None:
#     talker.match_results.add_good_match("Inhumans #1.cbz")
#     talker.match_results.good_matches.append("Inhumans #2.cbz")
#     talker.match_results.no_matches.append("Outsiders #1.cbz")
#     talker.match_results.no_matches.append("Outsiders #2.cbz")

#     expected_result = (
#         "\nSuccessful matches:\n------------------\nInhumans #1.cbz\nInhumans #2.cbz\n\n"
#         + "No matches:\n------------------\nOutsiders #1.cbz\nOutsiders #2.cbz\n"
#     )

#     talker._post_process_matches()
#     stdout, _ = capsys.readouterr()

#     assert stdout == expected_result



import zipfile
from argparse import ArgumentParser
from datetime import date
from pathlib import Path

import pytest
from darkseid.metadata import Basic, Credit, Metadata, Role, Series

from metrontagger.options import make_parser
from metrontagger.talker import Talker

CONTENT = "blah blah blah"


@pytest.fixture(scope="session")
def parser() -> ArgumentParser:
    return make_parser()


@pytest.fixture(scope="session")
def talker() -> Talker:
    username = "Foo"
    password = "Bar"
    return Talker(username, password)


@pytest.fixture(scope="function")
def fake_tpb_metadata() -> Metadata:
    meta_data = Metadata()
    meta_data.publisher = Basic("DC Comics")
    meta_data.series = Series("Batman", volume=1, format="Trade Paperback")
    meta_data.issue = "1"
    meta_data.cover_date = date(2021, 9, 1)
    meta_data.add_credit(Credit("Grant Morrison", [Role("Writer")]))
    meta_data.add_credit(Credit("Chris Burham", [Role("Artist"), Role("Cover")]))

    return meta_data


@pytest.fixture(scope="function")
def fake_metadata() -> Metadata:
    meta_data = Metadata()
    meta_data.publisher = Basic("DC Comics")
    meta_data.series = Series("Aquaman", volume=2)
    meta_data.issue = "1"
    meta_data.cover_date = date(2011, 9, 1)
    meta_data.add_credit(Credit("Peter David", [Role("Writer")]))
    meta_data.add_credit(Credit("Martin Egeland", [Role("Penciller"), Role("Cover")]))

    return meta_data


@pytest.fixture(scope="function")
def fake_comic(tmp_path_factory: Path) -> zipfile.ZipFile:
    test_dir = tmp_path_factory.mktemp("data")
    img_1 = test_dir / "image-1.jpg"
    img_1.write_text(CONTENT)
    img_2 = test_dir / "image-2.jpg"
    img_2.write_text(CONTENT)
    img_3 = test_dir / "image-3.jpg"
    img_3.write_text(CONTENT)

    z_file = tmp_path_factory.mktemp("comic") / "Aquaman v1 #001 (of 08) (1994).cbz"
    zf = zipfile.ZipFile(z_file, "w")
    try:
        zf.write(img_1)
        zf.write(img_2)
        zf.write(img_3)
    finally:
        zf.close()

    return z_file



"""Class to sort comic file based on it's metadata tags"""
import pathlib
from pathlib import Path
from shutil import Error, move
from typing import Optional, Tuple

import questionary
from darkseid.comic import Comic
from darkseid.metadata import Metadata

from metrontagger.styles import Styles
from metrontagger.utils import cleanup_string


class FileSorter:
    """Class to move comic files based on it's metadata tags"""

    def __init__(self, directory: str) -> None:
        self.sort_directory = directory

    @staticmethod
    def _cleanup_metadata(
        meta_data: Metadata,
    ) -> Tuple[Optional[str], Optional[str], Optional[str]]:
        """Clean the metadata string."""
        publisher = cleanup_string(meta_data.publisher.name)
        series = cleanup_string(meta_data.series.name)
        volume = cleanup_string(meta_data.series.volume)
        return publisher, series, volume

    @staticmethod
    def _move_files(orig: Path, new: Path) -> bool:
        try:
            move(orig, new)
            questionary.print(f"moved '{orig.name}' to '{new}'", style=Styles.SUCCESS)
            return True
        except Error:
            return False

    @staticmethod
    def _overwrite_existing(new_path: Path, old_comic: Path):
        existing_comic = new_path / old_comic.name
        if existing_comic.exists():
            MB_CONV = 1048576
            existing_size = existing_comic.stat().st_size / MB_CONV
            new_size = old_comic.stat().st_size / MB_CONV
            msg = f"{existing_comic.name} exists at {existing_comic.parent}.\nOld file: {existing_size:.2f} MB -> New file: {new_size:.2f} MB"
            questionary.print(msg, Styles.WARNING)
            if questionary.confirm(
                "Would you like to overwrite existing file?", default=False
            ).ask():
                existing_comic.unlink()

    def sort_comics(self, comic: Path) -> bool:
        """Method to move the comic file based on it's metadata tag"""
        comic_archive = Comic(comic)
        if comic_archive.has_metadata():
            meta_data = comic_archive.read_metadata()
        else:
            return False

        publisher, series, volume = self._cleanup_metadata(meta_data)

        if publisher and series and volume:
            tpb = meta_data.series.format == "Trade Paperback"
            new_path = (
                pathlib.Path(self.sort_directory)
                / publisher
                / f"{f'{series} TPB' if tpb else f'{series}'}"
                / f"v{volume}"
            )
        else:
            questionary.print(
                "Missing metadata from comic and will be unable to sort."
                + f"Publisher: {publisher}\nSeries: {series}\nVolume: {volume}",
                style=Styles.WARNING,
            )
            return False

        self._overwrite_existing(new_path, comic)

        if not new_path.is_dir():
            try:
                new_path.mkdir(parents=True)
            except PermissionError:
                questionary.print(
                    f"due to permission error, failed to create directory: {new_path}",
                    style=Styles.ERROR,
                )
                return False

        return self._move_files(comic, new_path)



def test_path_options(parser, tmpdir):
    parsed = parser.parse_args([str(tmpdir)])
    assert parsed.path == [str(tmpdir)]


def test_online_option(parser, tmpdir):
    parsed = parser.parse_args(["-o", str(tmpdir)])
    assert parsed.online is True
    assert parsed.path == [str(tmpdir)]


def test_id_option(parser, tmpdir):
    parsed = parser.parse_args(["--id", "1", str(tmpdir)])
    assert parsed.id == "1"
    assert parsed.path == [str(tmpdir)]


def test_delete_option(parser, tmpdir):
    parsed = parser.parse_args(["-d", str(tmpdir)])
    assert parsed.delete is True
    assert parsed.path == [str(tmpdir)]


def test_missing_option(parser, tmpdir):
    parsed = parser.parse_args(["--missing", str(tmpdir)])
    assert parsed.missing is True
    assert parsed.path == [str(tmpdir)]


def test_rename_option(parser, tmpdir):
    parsed = parser.parse_args(["-r", str(tmpdir)])
    assert parsed.rename is True
    assert parsed.path == [str(tmpdir)]


def test_ignore_option(parser, tmpdir):
    parsed = parser.parse_args(["--ignore-existing", str(tmpdir)])
    assert parsed.ignore_existing is True
    assert parsed.path == [str(tmpdir)]



from pathlib import Path
from typing import List

import questionary
from darkseid.comic import Comic
from darkseid.utils import get_recursive_filelist

from metrontagger.filerenamer import FileRenamer
from metrontagger.filesorter import FileSorter
from metrontagger.settings import MetronTaggerSettings
from metrontagger.styles import Styles
from metrontagger.talker import Talker


class Runner:
    """Main runner"""

    def __init__(self, config: MetronTaggerSettings) -> None:
        self.config = config

    def rename_comics(self, file_list: List[Path]) -> List[Path]:
        questionary.print(
            "\nStarting comic archive renaming:\n-------------------------------",
            style=Styles.TITLE,
        )

        # Lists to track filename changes
        new_file_names: List[Path] = []
        original_files_changed: List[Path] = []
        for comic in file_list:
            comic_archive = Comic(comic)
            if not comic_archive.has_metadata():
                questionary.print(
                    f"skipping '{comic.name}'. no metadata available.", style=Styles.WARNING
                )
                continue

            meta_data = comic_archive.read_metadata()
            renamer = FileRenamer(meta_data)
            renamer.set_template(self.config.rename_template)
            renamer.set_issue_zero_padding(self.config.rename_issue_number_padding)
            renamer.set_smart_cleanup(self.config.rename_use_smart_string_cleanup)

            unique_name = renamer.rename_file(comic)
            if unique_name is None:
                continue

            # track what files are being renamed
            new_file_names.append(unique_name)
            original_files_changed.append(comic)

            questionary.print(
                f"renamed '{comic.name}' -> '{unique_name.name}'", style=Styles.SUCCESS
            )

        # Update file_list for renamed files
        for original_file in original_files_changed:
            file_list.remove(original_file)

        # Add new file names to file list.
        file_list.extend(iter(new_file_names))
        return file_list

    def _export_to_cb7(self, file_list: List[Path]) -> None:
        questionary.print("\nExporting to cb7:\n-----------------", style=Styles.TITLE)
        for comic in file_list:
            ca = Comic(comic)
            if ca.is_zip() or ca.is_rar():
                new_fn = Path(comic).with_suffix(".cb7")
                if ca.export_as_cb7(new_fn):
                    questionary.print(
                        f"Exported '{comic.name}' to a cb7 archive.", style=Styles.SUCCESS
                    )
                    if self.config.delete_original:
                        questionary.print(f"Removing '{comic.name}'.", style=Styles.SUCCESS)
                        comic.unlink()
                else:
                    questionary.print(f"Failed to export '{comic.name}'", style=Styles.ERROR)
            else:
                questionary.print(
                    f"'{comic.name}' is not a cbr or cbz archive. skipping...",
                    style=Styles.WARNING,
                )

    def _export_to_zip(self, file_list: List[Path]) -> None:
        questionary.print("\nExporting to cbz:\n-----------------", style=Styles.TITLE)
        for comic in file_list:
            ca = Comic(comic)
            if ca.is_rar() or ca.is_sevenzip():
                new_fn = Path(comic).with_suffix(".cbz")
                if ca.export_as_zip(new_fn):
                    questionary.print(
                        f"Exported '{comic.name}' to a cbz archive.", style=Styles.SUCCESS
                    )
                    if self.config.delete_original:
                        questionary.print(f"Removing '{comic.name}'.", style=Styles.SUCCESS)
                        comic.unlink()
                else:
                    questionary.print(f"Failed to export '{comic.name}'", style=Styles.ERROR)
            else:
                questionary.print(
                    f"'{comic.name}' is not a cbr or cb7 archive. skipping...",
                    style=Styles.WARNING,
                )

    def _sort_comic_list(self, file_list: List[Path]) -> None:
        if not self.config.sort_dir:
            questionary.print(
                "\nUnable to sort files. No destination directory was provided.",
                style=Styles.ERROR,
            )
            return

        questionary.print(
            "\nStarting sorting of comic archives:\n----------------------------------",
            style=Styles.TITLE,
        )
        file_sorter = FileSorter(self.config.sort_dir)
        for comic in file_list:
            result = file_sorter.sort_comics(comic)
            if not result:
                questionary.print(f"unable to move {comic.name}.", style=Styles.ERROR)

    @staticmethod
    def _comics_with_no_metadata(file_list: List[Path]) -> None:
        questionary.print(
            "\nShowing files without metadata:\n-------------------------------",
            style=Styles.TITLE,
        )
        for comic in file_list:
            comic_archive = Comic(comic)
            if comic_archive.has_metadata():
                continue
            questionary.print(f"no metadata in '{comic.name}'", style=Styles.SUCCESS)

    @staticmethod
    def _delete_metadata(file_list: List[Path]) -> None:
        questionary.print("\nRemoving metadata:\n-----------------", style=Styles.TITLE)
        for comic in file_list:
            comic_archive = Comic(comic)
            if comic_archive.has_metadata():
                comic_archive.remove_metadata()
                questionary.print(
                    f"removed metadata from '{comic.name}'", style=Styles.SUCCESS
                )
            else:
                questionary.print(f"no metadata in '{comic.name}'", style=Styles.WARNING)

    def _has_credentials(self) -> bool:
        return bool(self.config.metron_user and self.config.metron_pass)

    def _get_credentials(self) -> bool:
        answers = questionary.form(
            user=questionary.text("What is your Metron username?"),
            passwd=questionary.text("What is your Metron password?"),
            save=questionary.confirm("Would you like to save your credentials?"),
        ).ask()
        if answers["user"] and answers["passwd"]:
            self.config.metron_user = answers["user"]
            self.config.metron_pass = answers["passwd"]
            if answers["save"]:
                self.config.save()
            return True
        return False

    def _get_sort_dir(self) -> bool:
        answers = questionary.form(
            dir=questionary.path("What directory should comics be sorted to?"),
            save=questionary.confirm("Would you like to save this location for future use?"),
        ).ask()
        if answers["dir"]:
            self.config.sort_dir = answers["dir"]
            if answers["save"]:
                self.config.save()
            return True
        return False

    def run(self) -> None:

        if not (file_list := get_recursive_filelist(self.config.path)):
            print("No files to process. Exiting.")
            exit(0)

        if self.config.missing:
            self._comics_with_no_metadata(file_list)

        if self.config.delete:
            self._delete_metadata(file_list)

        if self.config.id or self.config.online:
            if not self._has_credentials() and not self._get_credentials():
                questionary.print("No credentials provided. Exiting...")
                exit(0)

            t = Talker(self.config.metron_user, self.config.metron_pass)
            if self.config.id:
                if len(file_list) == 1:
                    t.retrieve_single_issue(file_list[0], self.config.id)
                else:
                    questionary.print(
                        "More than one file was passed for Id processing. Exiting...",
                        style=Styles.WARNING,
                    )
                    exit(0)
            else:
                t.identify_comics(file_list, self.config)

        if self.config.rename:
            file_list = self.rename_comics(file_list)

        if self.config.sort:
            if not self.config.sort_dir and not self._get_sort_dir():
                questionary.print("No sort directory given. Exiting...")
                exit(0)
            self._sort_comic_list(file_list)

        if self.config.export_to_cb7:
            self._export_to_cb7(file_list)

        if self.config.export_to_cbz:
            self._export_to_zip(file_list)



import sys
from pathlib import Path
from zipfile import ZipFile

import pytest
from darkseid.comic import Comic
from darkseid.metadata import Metadata

from metrontagger.filesorter import FileSorter


# Skip test for windows, until some with a windows box can help debug this.
@pytest.mark.skipif(sys.platform in ["win32"], reason="Skip Windows.")
def test_sort_comic_with_missing_metadata(
    fake_comic: ZipFile, fake_metadata: Metadata, tmp_path: Path
) -> None:
    test_dir = tmp_path / "sort10"

    fake_metadata.series.volume = None

    comic = Comic(fake_comic)
    if comic.has_metadata():
        comic.remove_metadata()
    comic.write_metadata(fake_metadata)
    file_sorter = FileSorter(test_dir)
    res = file_sorter.sort_comics(fake_comic)
    assert res is False


@pytest.mark.skipif(sys.platform in ["win32"], reason="Skip Windows.")
def test_sort_comic(fake_comic: ZipFile, fake_metadata: Metadata, tmp_path: Path) -> None:
    test_dir = tmp_path / "sort1"

    test_dir.mkdir()

    result_dir = (
        test_dir
        / fake_metadata.publisher.name
        / fake_metadata.series.name
        / f"v{fake_metadata.series.volume}"
    )

    # Write metadata to fake file
    comic = Comic(fake_comic)
    if comic.has_metadata():
        comic.remove_metadata()
    comic.write_metadata(fake_metadata)

    file_sorter = FileSorter(test_dir)
    res = file_sorter.sort_comics(fake_comic)
    assert result_dir.is_dir()
    assert res is True


@pytest.mark.skipif(sys.platform in ["win32"], reason="Skip Windows.")
def test_sort_tpb(fake_comic: ZipFile, fake_tpb_metadata: Metadata, tmp_path: Path) -> None:
    test_dir = tmp_path / "sort1"

    test_dir.mkdir()

    result_dir = (
        test_dir
        / fake_tpb_metadata.publisher.name
        / f"{fake_tpb_metadata.series.name} TPB"
        / f"v{fake_tpb_metadata.series.volume}"
    )

    # Write metadata to fake file
    comic = Comic(fake_comic)
    if comic.has_metadata():
        comic.remove_metadata()
    comic.write_metadata(fake_tpb_metadata)

    file_sorter = FileSorter(test_dir)
    res = file_sorter.sort_comics(fake_comic)
    assert result_dir.is_dir()
    assert res is True


def test_sort_files_without_metadata(fake_comic: ZipFile, tmp_path: Path) -> None:
    test_dir = tmp_path / "sort2"
    # If we add more tests we should probably create another tmpfile
    # since we are removing the metadata from the tmpfile
    comic = Comic(fake_comic)
    comic.remove_metadata()

    file_sorter = FileSorter(test_dir)
    res = file_sorter.sort_comics(fake_comic)
    assert res is False



import sys
from typing import List
from zipfile import ZipFile

import pytest
from darkseid.comic import Comic
from darkseid.metadata import Basic
from mokkari.issue import IssueSchema, IssuesList
from mokkari.session import Session

from metrontagger.talker import Talker


@pytest.fixture()
def metron_response():
    i = {
        "id": 31047,
        "publisher": {"id": 1, "name": "Marvel"},
        "series": {
            "id": 2222,
            "name": "The Spectacular Spider-Man",
            "sort_name": "Spectacular Spider-Man",
            "volume": 1,
            "series_type": {"id": 2, "name": "Cancelled Series"},
            "genres": [{"id": 1, "name": "Super-Hero"}],
        },
        "number": "47",
        "title": "",
        "name": ["A Night on the Prowl!"],
        "cover_date": "1980-10-01",
        "store_date": None,
        "price": "0.50",
        "sku": "",
        "isbn": "",
        "upc": "",
        "page": 36,
        "desc": "Spider-Man goes on a wild goose chase to find out who is behind the Prowler impersonation.",
        "image": "https://static.metron.cloud/media/issue/2021/05/22/the-spectacular-spider-man-47.jpg",
        "arcs": [],
        "credits": [
            {"id": 233, "creator": "Al Milgrom", "role": [{"id": 7, "name": "Cover"}]},
            {
                "id": 1402,
                "creator": "Bruce Patterson",
                "role": [{"id": 4, "name": "Inker"}, {"id": 6, "name": "Letterer"}],
            },
            {"id": 128, "creator": "Dennis O'Neil", "role": [{"id": 8, "name": "Editor"}]},
            {"id": 1035, "creator": "Glynis Oliver", "role": [{"id": 5, "name": "Colorist"}]},
            {
                "id": 624,
                "creator": "Jim Shooter",
                "role": [{"id": 20, "name": "Editor In Chief"}],
            },
            {"id": 675, "creator": "Marie Severin", "role": [{"id": 3, "name": "Penciller"}]},
            {"id": 273, "creator": "Roger Stern", "role": [{"id": 1, "name": "Writer"}]},
        ],
        "characters": [
            {
                "id": 6784,
                "name": "Debra Whitman",
                "modified": "2021-05-22T22:47:25.294935-04:00",
            },
            {
                "id": 3067,
                "name": "Hobgoblin (Kingsley)",
                "modified": "2019-12-11T17:21:51.365470-05:00",
            },
            {"id": 2415, "name": "Prowler", "modified": "2019-08-24T16:37:06.997976-04:00"},
            {"id": 145, "name": "Spider-Man", "modified": "2022-05-16T09:22:42.644589-04:00"},
        ],
        "teams": [{"id": 1, "name": "Bar Foo"}],
        "reprints": [
            {"id": 2860, "issue": "Twilight (1990) #1"},
            {"id": 2861, "issue": "Twilight (1990) #2"},
            {"id": 2862, "issue": "Twilight (1990) #3"},
        ],
        "variants": [],
        "rating": {"id": 1, "name": "Mature"},
        "resource_url": "https://metron.cloud/issue/american-vampire-1976-2020-2/",
        "modified": "2022-05-29T08:22:38.584485-04:00",
    }
    return IssueSchema().load(i)


def create_reprint_list(resource) -> List[Basic]:
    return [Basic(r.issue, r.id) for r in resource]


def create_resource_list(resource) -> List[Basic]:
    return [Basic(r.name, r.id) for r in resource]


def create_read_md_resource_list(resource) -> List[Basic]:
    return [Basic(r.name) for r in resource]


def create_read_md_story(resource) -> List[Basic]:
    return [Basic(story) for story in resource]


def test_map_resp_to_metadata(talker: Talker, metron_response) -> None:
    md = talker._map_resp_to_metadata(metron_response)
    assert md is not None
    assert md.stories == create_read_md_story(metron_response.story_titles)
    assert md.series.name == metron_response.series.name
    assert md.series.volume == metron_response.series.volume
    assert md.publisher.name == metron_response.publisher.name
    assert md.issue == metron_response.number
    assert md.story_arcs == create_resource_list(metron_response.arcs)
    assert md.teams == create_resource_list(metron_response.teams)
    assert md.cover_date.year == metron_response.cover_date.year
    assert md.characters == create_resource_list(metron_response.characters)
    assert md.credits is not None
    assert md.credits[0].person == "Al Milgrom"
    assert md.credits[0].role[0].name == "Cover"
    assert md.reprints == create_reprint_list(metron_response.reprints)
    assert md.age_rating == "Mature 17+"
    assert md.web_link == metron_response.resource_url


def test_map_resp_to_metadata_with_no_story_name(talker: Talker, metron_response) -> None:
    test_data = metron_response
    test_data.story_titles = None
    meta_data = talker._map_resp_to_metadata(test_data)
    assert meta_data is not None
    assert len(meta_data.stories) == 0
    assert meta_data.series.name == metron_response.series.name
    assert meta_data.series.volume == metron_response.series.volume
    assert meta_data.publisher.name == metron_response.publisher.name
    assert meta_data.issue == metron_response.number
    assert meta_data.cover_date.year == metron_response.cover_date.year


@pytest.fixture()
def issue_list_response() -> IssuesList:
    i = {
        "count": 8,
        "next": None,
        "previous": None,
        "results": [
            {"id": 3634, "__str__": "Aquaman #1", "cover_date": "1962-02-01"},
            {"id": 2471, "__str__": "Aquaman #1", "cover_date": "1986-02-01"},
            {"id": 2541, "__str__": "Aquaman #1", "cover_date": "1989-06-01"},
            {"id": 2510, "__str__": "Aquaman #1", "cover_date": "1991-12-01"},
            {"id": 1776, "__str__": "Aquaman #1", "cover_date": "1994-08-01"},
            {"id": 2429, "__str__": "Aquaman #1", "cover_date": "2003-02-01"},
            {"id": 2523, "__str__": "Aquaman #1", "cover_date": "2011-11-01"},
            {"id": 2896, "__str__": "Aquaman #1", "cover_date": "2016-08-01"},
        ],
    }
    return IssuesList(i)


def test_process_file(
    talker: Talker, fake_comic: ZipFile, issue_list_response: IssuesList, mocker
) -> None:
    # Remove any existing metadata from comic fixture
    ca = Comic(fake_comic)
    if ca.has_metadata():
        ca.remove_metadata()

    # Mock the call to Metron
    mocker.patch.object(Session, "issues_list", return_value=issue_list_response)
    talker._process_file(fake_comic, False)

    id, multiple = talker._process_file(fake_comic, False)
    assert id is None
    assert multiple
    assert fake_comic in [c.filename for c in talker.match_results.multiple_matches]

    id_list = []
    for c in talker.match_results.multiple_matches:
        id_list.extend(i.id for i in c.matches)
    assert 2471 in id_list


@pytest.mark.skipif(sys.platform in ["win32"], reason="Skip Windows.")
def test_write_issue_md(talker: Talker, fake_comic: ZipFile, metron_response, mocker) -> None:
    # Remove any existing metadata from comic fixture
    ca = Comic(fake_comic)
    if ca.has_metadata():
        ca.remove_metadata()

    # Mock the call to Metron
    mocker.patch.object(Session, "issue", return_value=metron_response)
    talker.retrieve_single_issue(fake_comic, 5)

    # Now let's test writing the metadata to file
    talker._write_issue_md(fake_comic, 1)
    ca = Comic(fake_comic)
    assert ca.has_metadata()
    ca_md = ca.read_metadata()
    assert ca_md.stories == create_read_md_story(metron_response.story_titles)
    assert ca_md.series.name == metron_response.series.name
    assert ca_md.series.volume == metron_response.series.volume
    assert ca_md.publisher.name == metron_response.publisher.name
    assert ca_md.issue == metron_response.number
    assert ca_md.teams == create_read_md_resource_list(metron_response.teams)
    assert ca_md.cover_date.year == metron_response.cover_date.year
    assert ca_md.characters == create_read_md_resource_list(metron_response.characters)
    assert ca_md.credits is not None
    assert ca_md.credits[0].person == "Roger Stern"
    assert ca_md.credits[0].role[0].name == "Writer"


@pytest.mark.skipif(sys.platform in ["win32"], reason="Skip Windows.")
def test_retrieve_single_issue(
    talker: Talker, fake_comic: ZipFile, metron_response, mocker
) -> None:
    # Remove any existing metadata from comic fixture
    ca = Comic(fake_comic)
    if ca.has_metadata():
        ca.remove_metadata()

    # Mock the call to Metron
    mocker.patch.object(Session, "issue", return_value=metron_response)
    talker.retrieve_single_issue(fake_comic, 10)

    # Now let's test the metadata
    ca = Comic(fake_comic)
    assert ca.has_metadata()
    ca_md = ca.read_metadata()
    assert ca_md.stories == create_read_md_story(metron_response.story_titles)
    assert ca_md.series.name == metron_response.series.name
    assert ca_md.series.volume == metron_response.series.volume
    assert ca_md.publisher.name == metron_response.publisher.name
    assert ca_md.issue == metron_response.number
    assert ca_md.teams == create_read_md_resource_list(metron_response.teams)
    assert ca_md.cover_date.year == metron_response.cover_date.year
    assert ca_md.characters == create_read_md_resource_list(metron_response.characters)
    assert ca_md.credits is not None
    assert ca_md.credits[0].person == "Roger Stern"
    assert ca_md.credits[0].role[0].name == "Writer"



from datetime import date
from zipfile import ZipFile

from darkseid.metadata import Metadata, Series

from metrontagger.filerenamer import FileRenamer


def test_determine_name(fake_comic: ZipFile, fake_metadata: Metadata) -> None:
    expected_result = "Aquaman v2 #001 (2011).cbz"

    renamer = FileRenamer(fake_metadata)
    new_file_name = renamer.determine_name(fake_comic)
    assert new_file_name == expected_result


def test_rename_file(fake_comic: ZipFile) -> None:
    md = Metadata()
    md.series = Series("Batman", volume=2)
    md.issue = "100"
    md.cover_date = date(2020, 9, 1)

    # Verify what the original name of the file is.
    assert fake_comic.name == "Aquaman v1 #001 (of 08) (1994).cbz"

    renamer = FileRenamer(md)
    renamed_file = renamer.rename_file(fake_comic)
    assert renamed_file is not None

    expected_result = (
        f"{md.series.name} v{md.series.volume} #{md.issue} ({md.cover_date.year}).cbz"
    )
    assert expected_result == renamed_file.name


def test_empty_parenthesis(fake_metadata: Metadata) -> None:
    test_str = "Aquaman #1()"

    rn = FileRenamer(fake_metadata)
    res = rn._remove_empty_separators(test_str)
    assert res == "Aquaman #1"



"""Cli for Metron-Tagger."""

from argparse import Namespace

from metrontagger.options import make_parser
from metrontagger.run import Runner
from metrontagger.settings import MetronTaggerSettings


def get_args() -> Namespace:
    parser = make_parser()
    return parser.parse_args()


def get_configs(opts: Namespace) -> MetronTaggerSettings:
    config = MetronTaggerSettings()
    if opts.path:
        config.path = opts.path

    if opts.id:
        config.id = opts.id

    if opts.online:
        config.online = opts.online

    if opts.missing:
        config.missing = opts.missing

    if opts.delete:
        config.delete = opts.delete

    if opts.rename:
        config.rename = opts.rename

    if opts.sort:
        config.sort = opts.sort

    if opts.interactive:
        config.interactive = opts.interactive

    if opts.ignore_existing:
        config.ignore_existing = opts.ignore_existing

    if opts.export_to_cb7:
        config.export_to_cb7 = opts.export_to_cb7

    if opts.export_to_cbz:
        config.export_to_cbz = opts.export_to_cbz

    if opts.delete_original:
        config.delete_original = opts.delete_original

    return config


def main():
    args = get_args()
    config = get_configs(args)

    runner = Runner(config)
    runner.run()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['metrontagger']

package_data = \
{'': ['*']}

install_requires = \
['darkseid>=2.0.1,<3.0.0',
 'mokkari>=2.3.3,<3.0.0',
 'pyxdg>=0.28,<0.29',
 'questionary>=1.10.0,<2.0.0']

entry_points = \
{'console_scripts': ['metron-tagger = metrontagger.cli:main']}

setup_kwargs = {
    'name': 'metron-tagger',
    'version': '1.4.3',
    'description': 'A program to write metadata from metron.cloud to a comic archive',
    'long_description': '=============\nMetron-Tagger\n=============\n\n.. image:: https://img.shields.io/pypi/v/metron-tagger.svg\n    :target: https://pypi.org/project/metron-tagger/\n\n.. image:: https://img.shields.io/pypi/pyversions/metron-tagger.svg\n    :target: https://pypi.org/project/metron-tagger/\n\n.. image:: https://img.shields.io/github/license/bpepple/metron-tagger\n    :target: https://opensource.org/licenses/GPL-3.0\n\n.. image:: https://codecov.io/gh/Metron-Project/metron-tagger/branch/master/graph/badge.svg?token=d8TyzWM2Uz \n    :target: https://codecov.io/gh/Metron-Project/metron-tagger\n\n.. image:: https://img.shields.io/badge/code%20style-black-000000.svg\n    :target: https://github.com/psf/black\n\nQuick Description\n-----------------\n\nA command-line tool to tag comic archives with metadata from metron.cloud_.\n\n.. _metron.cloud: https://metron.cloud\n\nInstallation\n------------\n\nPyPi\n~~~~\n\nOr install it yourself:\n\n.. code:: bash\n\n  $ pip install --user metron-tagger\n\nFAQ\n---\n\n**How to enable RAR support?**\n\n- It depends on the unrar command-line utility, and expects it to be in your $PATH.\n\nHelp\n----\n\n::\n\n  usage: metron-tagger [-h] [-r] [-o] [--id ID] [-d] [--ignore-existing] [-i] [--missing] [-s] [-e] [-z] [--delete-original] [--version] path [path ...]\n\n  Read in a file or set of files, and return the result.\n\n  positional arguments:\n    path                 Path of a file or a folder of files.\n\n  options:\n    -h, --help           show this help message and exit\n    -r, --rename         Rename comic archive from the files metadata. (default: False)\n    -o, --online         Search online and attempt to identify comic archive. (default: False)\n    --id ID              Identify file for tagging with the Metron Issue Id. (default: None)\n    -d, --delete         Delete the metadata tags from the file. (default: False)\n    --ignore-existing    Ignore files that have existing metadata tag. (default: False)\n    -i, --interactive    Interactively query the user when there are matches for an online search. (default: False)\n    --missing            List files without metadata. (default: False)\n    -s, --sort           Sort files that contain metadata tags. (default: False)\n    -e, --export-to-cb7  Export a CBZ (zip) or CBR (rar) archive to a CB7 (7zip) archive. (default: False)\n    -z, --export-to-cbz  Export a CB7 (7zip) or CBR (rar) archive to a CBZ (zip) archive. (default: False)\n    --delete-original    Delete the original archive after successful export to another format. (default: False)\n    --version            Show the version number and exit\n\n\nBugs/Requests\n-------------\n\nPlease use the `GitHub issue tracker <https://github.com/Metron-Project/metron-tagger/issues>`_ to submit bugs or request features.\n\nLicense\n-------\n\nThis project is licensed under the `GPLv3 License <LICENSE>`_.\n\n',
    'author': 'Brian Pepple',
    'author_email': 'bdpepple@gmail.com',
    'maintainer': 'Brian Pepple',
    'maintainer_email': 'bdpepple@gmail.com',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



"""Class to handle project settings"""
import configparser
import platform
from os import environ
from pathlib import Path, PurePath
from typing import Optional

import questionary
from xdg.BaseDirectory import save_config_path

from metrontagger.styles import Styles


class MetronTaggerSettings:
    """Class to handle project settings"""

    @staticmethod
    def get_settings_folder() -> Path:
        """Method to determine where the users settings should be saved"""

        if platform.system() != "Windows":
            return Path(save_config_path("metron-tagger"))

        windows_path = PurePath(environ["APPDATA"]).joinpath("MetronTagger")
        return Path(windows_path)

    def _migrate_old_config(self) -> None:
        old_config = Path.home() / ".MetronTagger" / "settings.ini"
        if old_config.exists():
            # Let's move any existing config to the new location
            old_config.replace(self.settings_file)
            questionary.print(
                f"Migrated existing configuration to: {self.settings_file.parent}",
                style=Styles.WARNING,
            )
            if old_config.parent.exists():
                questionary.print(
                    f"Removing old configuration directory: {old_config.parent}",
                    style=Styles.WARNING,
                )
                old_config.parent.rmdir()

    def __init__(self, config_dir: Optional[str] = None) -> None:
        # Metron creditials
        self.metron_user: str = ""
        self.metron_pass: str = ""
        self.path: str = ""
        self.sort_dir: str = ""
        self.id: bool = False
        self.online: bool = False
        self.missing: bool = False
        self.delete: bool = False
        self.rename: bool = False
        self.sort: bool = False
        self.interactive: bool = False
        self.ignore_existing: bool = False
        self.export_to_cb7: bool = False
        self.export_to_cbz: bool = False
        self.delete_original: bool = False

        # Rename settings
        self.rename_template = "%series% v%volume% #%issue% (%year%)"
        self.rename_issue_number_padding = 3
        self.rename_use_smart_string_cleanup = True

        self.config = configparser.RawConfigParser()

        folder = Path(config_dir) if config_dir else MetronTaggerSettings.get_settings_folder()
        self.settings_file = folder / "settings.ini"

        if not self.settings_file.parent.exists():
            self.settings_file.parent.mkdir()

        # We can probably remove this in a year or so. Around 2023-06-01
        self._migrate_old_config()

        # Write the config file if it doesn't exist
        if not self.settings_file.exists():
            self.save()
        else:
            self.load()

    def load(self) -> None:
        """Method to retrieve a users settings"""
        self.config.read(self.settings_file)

        if self.config.has_option("metron", "user"):
            self.metron_user = self.config["metron"]["user"]

        if self.config.has_option("metron", "password"):
            self.metron_pass = self.config["metron"]["password"]

        if self.config.has_option("DEFAULT", "sort_dir"):
            self.sort_dir = self.config["DEFAULT"]["sort_dir"]

        if self.config.has_option("rename", "rename_template"):
            self.rename_template = self.config.get("rename", "rename_template")

        if self.config.has_option("rename", "rename_issue_number_padding"):
            self.rename_issue_number_padding = self.config.getint(
                "rename", "rename_issue_number_padding"
            )

        if self.config.has_option("rename", "rename_use_smart_string_cleanup"):
            self.rename_use_smart_string_cleanup = self.config.getboolean(
                "rename", "rename_use_smart_string_cleanup"
            )

    def save(self) -> None:
        """Method to save a users settings"""
        if not self.config.has_section("metron"):
            self.config.add_section("metron")

        self.config["DEFAULT"]["sort_dir"] = self.sort_dir
        self.config["metron"]["user"] = self.metron_user
        self.config["metron"]["password"] = self.metron_pass

        if not self.config.has_section("rename"):
            self.config.add_section("rename")

        self.config.set("rename", "rename_template", self.rename_template)
        self.config.set(
            "rename", "rename_issue_number_padding", str(self.rename_issue_number_padding)
        )
        self.config.set(
            "rename",
            "rename_use_smart_string_cleanup",
            str(self.rename_use_smart_string_cleanup),
        )

        with self.settings_file.open("w") as configfile:
            self.config.write(configfile)



from datetime import datetime
from pathlib import Path
from typing import List, Optional, Tuple

import mokkari
import questionary
from darkseid.comic import Comic
from darkseid.issue_string import IssueString
from darkseid.metadata import Basic, Credit, Metadata, Role, Series
from mokkari.issue import CreditsSchema, IssueSchema, RolesSchema

from metrontagger import __version__
from metrontagger.settings import MetronTaggerSettings
from metrontagger.styles import Styles
from metrontagger.utils import create_query_params


class MultipleMatch:
    """Class to hold information on searches with multiple matches"""

    def __init__(self, filename: Path, match_list: List[IssueSchema]) -> None:
        self.filename = filename
        self.matches = match_list


class OnlineMatchResults:
    """Class to track online match results"""

    def __init__(self) -> None:
        self.good_matches: List[Path] = []
        self.no_matches: List[Path] = []
        self.multiple_matches: List[MultipleMatch] = []

    def add_good_match(self, file_name: Path) -> None:
        self.good_matches.append(file_name)

    def add_no_match(self, file_name: Path) -> None:
        self.no_matches.append(file_name)

    def add_multiple_match(self, multi_match: MultipleMatch) -> None:
        self.multiple_matches.append(multi_match)


class Talker:
    def __init__(self, username: str, password: str) -> None:
        self.api = mokkari.api(username, password)
        self.match_results = OnlineMatchResults()

    @staticmethod
    def _create_choice_list(match_set: List[IssueSchema]) -> List[questionary.Choice]:
        issue_lst = []
        for i in match_set:
            c = questionary.Choice(title=f"{i.issue_name} ({i.cover_date})", value=i.id)
            issue_lst.append(c)

        issue_lst.append(questionary.Choice(title="Skip", value=""))
        return issue_lst

    def _select_choice_from_matches(
        self, fn: Path, match_set: List[IssueSchema]
    ) -> Optional[int]:
        """
        Function to ask user to choice which issue metadata to write,
        when there are multiple choices
        """
        questionary.print(f"\n{fn.name} - Results found:", style=Styles.TITLE)

        # sort match list by cover date
        match_set = sorted(match_set, key=lambda m: m.cover_date)
        choices = self._create_choice_list(match_set)

        return questionary.select("Select an issue to match", choices=choices).ask()

    def _process_file(self, fn: Path, interactive: bool) -> Tuple[Optional[int], bool]:
        ca = Comic(fn)

        if not ca.is_writable() and not ca.seems_to_be_a_comic_archive():
            questionary.print(
                f"{fn.name} appears not to be a comic or writable.", style=Styles.ERROR
            )
            return None, False

        params = create_query_params(fn)
        i_list = self.api.issues_list(params=params)
        result_count = len(i_list)

        issue_id = None
        multiple_match = False
        if result_count <= 0:
            issue_id = None
            self.match_results.add_no_match(fn)
            multiple_match = False
        elif result_count > 1:
            issue_id = None
            self.match_results.add_multiple_match(MultipleMatch(fn, i_list))
            multiple_match = True
        elif result_count == 1:
            if not interactive:
                issue_id = i_list[0].id
                self.match_results.add_good_match(fn)
            else:
                issue_id = self._select_choice_from_matches(fn, i_list)
                if issue_id:
                    self.match_results.add_good_match(fn)
            multiple_match = False

        return issue_id, multiple_match

    def _post_process_matches(self) -> None:
        # Print file matching results.
        if self.match_results.good_matches:
            questionary.print("\nSuccessful matches:\n------------------", style=Styles.TITLE)
            for comic in self.match_results.good_matches:
                questionary.print(f"{comic}", style=Styles.SUCCESS)

        if self.match_results.no_matches:
            questionary.print("\nNo matches:\n------------------", style=Styles.TITLE)
            for comic in self.match_results.no_matches:
                questionary.print(f"{comic}", style=Styles.WARNING)

        # Handle files with multiple matches
        if self.match_results.multiple_matches:
            for match_set in self.match_results.multiple_matches:
                if issue_id := self._select_choice_from_matches(
                    match_set.filename, match_set.matches
                ):
                    self._write_issue_md(match_set.filename, issue_id)

    def _write_issue_md(self, filename: Path, issue_id: int) -> None:
        success = False

        if resp := self.api.issue(issue_id):
            ca = Comic(filename)
            meta_data = Metadata()
            meta_data.set_default_page_list(ca.get_number_of_pages())
            md = self._map_resp_to_metadata(resp)
            md.overlay(meta_data)
            success = ca.write_metadata(md)

        if success:
            questionary.print(f"Match found for '{filename.name}'.", style=Styles.SUCCESS)
        else:
            questionary.print(
                f"There was a problem writing metadata for '{filename.name}'.",
                style=Styles.ERROR,
            )

    def identify_comics(self, file_list: List[Path], config: MetronTaggerSettings):
        questionary.print(
            "\nStarting online search and tagging:\n----------------------------------",
            style=Styles.TITLE,
        )

        for fn in file_list:
            if config.ignore_existing:
                comic_archive = Comic(fn)
                if comic_archive.has_metadata():
                    questionary.print(
                        f"{fn.name} has metadata. Skipping...", style=Styles.WARNING
                    )
                    continue

            issue_id, multiple_match = self._process_file(fn, config.interactive)
            if issue_id:
                self._write_issue_md(fn, issue_id)
            elif not multiple_match:
                questionary.print(f"No Match for '{fn.name}'.", style=Styles.ERROR)

        # Print match results
        self._post_process_matches()

    def retrieve_single_issue(self, fn: Path, id: int) -> None:
        self._write_issue_md(fn, id)

    @staticmethod
    def _map_resp_to_metadata(resp: IssueSchema) -> Metadata:  # NOQA C901
        # Helper functions
        def create_resource_list(resource) -> List[Basic]:
            return [Basic(r.name, r.id) for r in resource]

        def create_note(issue_id: int) -> str:
            now_date = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            return f"Tagged with MetronTagger-{__version__} using info from Metron on {now_date}. [issue_id:{issue_id}]"

        def add_credits_to_metadata(
            md: Metadata, credits_resp: List[CreditsSchema]
        ) -> Metadata:
            def create_role_list(roles: List[RolesSchema]) -> List[Role]:
                return [Role(r.name, r.id) for r in roles]

            for c in credits_resp:
                if c.role:
                    md.add_credit(Credit(c.creator, create_role_list(c.role), c.id))
                else:
                    md.add_credit(Credit(c.creator, [], c.id))
            return md

        def map_ratings(rating: str) -> str:
            age_rating = rating.lower()
            if age_rating in {"everyone", "cca"}:
                return "Everyone"
            elif age_rating in {"teen", "teen plus"}:
                return "Teen"
            elif age_rating == "mature":
                return "Mature 17+"
            else:
                return "Unknown"

        md = Metadata()
        md.info_source = Basic("Metron", resp.id)
        md.series = Series(
            resp.series.name,
            resp.series.id,
            resp.series.sort_name,
            resp.series.volume,
            resp.series.series_type.name,
        )
        md.issue = IssueString(resp.number).as_string()
        md.publisher = Basic(resp.publisher.name, resp.publisher.id)
        md.cover_date = resp.cover_date
        md.comments = resp.desc
        md.notes = create_note(md.info_source.id_)
        if resp.story_titles:
            md.stories = [Basic(story) for story in resp.story_titles]
        if resp.characters:
            md.characters = create_resource_list(resp.characters)
        if resp.teams:
            md.teams = create_resource_list(resp.teams)
        if resp.arcs:
            md.story_arcs = create_resource_list(resp.arcs)
        if resp.series.genres:
            md.genres = create_resource_list(resp.series.genres)
        if resp.reprints:
            md.reprints = [Basic(r.issue, r.id) for r in resp.reprints]
        if resp.credits:
            md = add_credits_to_metadata(md, resp.credits)
        if resp.rating:
            md.age_rating = map_ratings(resp.rating.name)
        if resp.resource_url:
            md.web_link = resp.resource_url

        return md
